###-----------------------------------------------------------------------------
### example: build_mcu_fpga_pynq_zcu104.tcl
### A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
###
### Contributors
###
### David Flynn (d.w.flynn@soton.ac.uk)
###
### Copyright � 2022, SoC Labs (www.soclabs.org)
###-----------------------------------------------------------------------------
#
# developed & tested using vivado_version 2021.1
#
# usage:
# vivado -mode tcl -source scripts/build_mcu_fpga_pynq_zcu104.tcl
# vivado -mode gui -source scripts/build_mcu_fpga_pynq_zcu104.tcl

# for ARM MPS3 as target
set xilinx_part xcku115-flvb1760-1-c
set project project_arm_mps3
set importDir target_fpga_arm_mps3
set ipDir ./ip_repo
set mcuDir ./vivado/built_mcu_fpga/MCULIB
set outDir ./MPS_FPGA_build

#
# STEP#0: build the cm0sdk design (without pads) as an "IP" library component for the testbench (in MCULIB)
#
source scripts/build_mcu_fpga_ip.tcl

#
# STEP#1: setup design sources and constraints
#
set_part $xilinx_part
set_property TARGET_LANGUAGE Verilog [current_project]
set_property DEFAULT_LIB work [current_project]

set paths [list \
 $ipDir\
 $mcuDir\
 ]

# Set IP repository paths
set obj [get_filesets sources_1]
if { $obj != {} } {
   set_property "ip_repo_paths" "[file normalize $ipDir] [file normalize $mcuDir]" $obj
   # Rebuild user ip_repo's index before adding any source files
   update_ip_catalog -rebuild
}

report_ip_status

#
# STEP#2: create Block Diagram and add specific IO wrapper (and import matching pinmap.xdc)
#
# using script written out from GUI capture

create_bd_design design_1

read_verilog $importDir/design_1_wrapper.v
source $importDir/design_1.tcl
create_root_design ""

add_files -norecurse -scan_for_includes {../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0/logical/cmsdk_apb_dualtimers/verilog/cmsdk_apb_dualtimers_defs.v ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0/logical/cmsdk_apb_watchdog/verilog/cmsdk_apb_watchdog_defs.v ../verilog/pl230_defs.v}
set_property is_global_include true [get_files  ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0/logical/cmsdk_apb_dualtimers/verilog/cmsdk_apb_dualtimers_defs.v]
set_property is_global_include true [get_files  ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0/logical/cmsdk_apb_watchdog/verilog/cmsdk_apb_watchdog_defs.v]
set_property is_global_include true [get_files  ../verilog/pl230_defs.v]

set_property file_type {Verilog Header} [get_files  ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0/logical/cmsdk_apb_dualtimers/verilog/cmsdk_apb_dualtimers_defs.v]
set_property file_type {Verilog Header} [get_files  ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0/logical/cmsdk_apb_watchdog/verilog/cmsdk_apb_watchdog_defs.v]
set_property file_type {Verilog Header} [get_files  ../verilog/pl230_defs.v]

add_files $importDir/fpga_pinmap.xdc

set_property top design_1_wrapper [current_fileset]

#
# STEP#3: save in Project mode to complete flow
#

save_project_as $project ./$project -exclude_run_results -force

update_compile_order -fileset sources_1

#
# STEP#4: synthesize project
#
set_property part $xilinx_part [get_runs synth_1]
launch_runs synth_1 -jobs 8

wait_on_run synth_1

#
# STEP#5: place and route project
#
set_property part $xilinx_part [get_runs impl_1]
launch_runs impl_1 -to_step write_bitstream -jobs 8

wait_on_run impl_1

#
# STEP#6: export design_1.bit and design_1.hwh files for PYNQ
#

write_hw_platform -fixed -include_bit -force -file $project/design_1.xsa

exec unzip -u -o $project/design_1.xsa -d $project/export
exec mkdir -p $outDir
exec cp -p $project/export/design_1.bit $outDir
exec cp -p $project/export/design_1.hwh $outDir

#exec rm -Rf vivado/

exit 1
