###-----------------------------------------------------------------------------
### example: build_mcu_fpga_pynq_zcu104.tcl
### A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
###
### Contributors
###
### David Flynn (d.w.flynn@soton.ac.uk)
###
### Copyright (C) 2022-3 SoC Labs (www.soclabs.org)
###-----------------------------------------------------------------------------
#
# developed & tested using vivado_version 2021.1
#
# usage:
# vivado -mode tcl -source $SOCLABS_FPGA_IMP_PATH/scripts/build_mcu_fpga_pynq_zcu104.tcl
# vivado -mode gui -source $SOCLABS_FPGA_IMP_PATH/scripts/build_mcu_fpga_pynq_zcu104.tcl

### Design specific
set proj_dir    $::env(SOCLABS_PROJECT_DIR)
set fpga_impl   $::env(SOCLABS_FPGA_IMP_PATH)
set AAA_ip      $::env(ARM_IP_LIBRARY_PATH)/latest
set nanosoc_sys $::env(SOCLABS_NANOSOC_TECH_DIR)/system

### Target specific
# for Xilinx ZCU104 as PYNQ target
set xilinx_part xczu7ev-ffvc1156-2-e
set project project_pynq_zcu104
set fpga_target $fpga_impl/target_fpga_zcu104
set ip_repo $fpga_impl/ip_repo
set mcu_lib ./built_mcu_fpga/MCULIB
set pynqDir ./pynq_export/pz104/pynq/overlays/soclabs
set exportDir /research/soclabs/pynq_export/pz104/pynq/overlays/soclabs
#set_property BOARD_PART xilinx.com:zcu104:part0:1.1 [current_project]

#
# STEP#0: build the cm0sdk design (without pads) as an "IP" library component for the testbench (in MCULIB)
#
source $fpga_impl/scripts/build_mcu_fpga_ip.tcl

#
# STEP#1: setup design sources and constraints
#
set_part $xilinx_part
set_property TARGET_LANGUAGE Verilog [current_project]
set_property DEFAULT_LIB work [current_project]

set paths [list \
 $ip_repo\
 $mcu_lib\
 ]

# Set IP repository paths
set obj [get_filesets sources_1]
if { $obj != {} } {
   set_property "ip_repo_paths" "[file normalize $ip_repo] [file normalize $mcu_lib]" $obj
   # Rebuild user ip_repo's index before adding any source files
   update_ip_catalog -rebuild
}

report_ip_status

#
# STEP#2: create Block Diagram and add specific IO wrapper (and import matching pinmap.xdc)
#
# using script written out from GUI capture

create_bd_design design_1

read_verilog $fpga_target/design_1_wrapper.v
source $fpga_target/design_1.tcl
create_root_design ""

set arm_ip_lib    $::env(ARM_IP_LIBRARY_PATH)/latest
add_files -norecurse -scan_for_includes "$arm_ip_lib/Corstone-101/logical/cmsdk_apb_dualtimers/verilog/cmsdk_apb_dualtimers_defs.v $arm_ip_lib/Corstone-101/logical/cmsdk_apb_watchdog/verilog/cmsdk_apb_watchdog_defs.v $proj_dir/system/defines/pl230/pl230_defs.v"
set_property is_global_include true [get_files  $arm_ip_lib/Corstone-101/logical/cmsdk_apb_dualtimers/verilog/cmsdk_apb_dualtimers_defs.v]
set_property is_global_include true [get_files  $arm_ip_lib/Corstone-101/logical/cmsdk_apb_watchdog/verilog/cmsdk_apb_watchdog_defs.v]
set_property is_global_include true [get_files  $proj_dir/system/defines/pl230/pl230_defs.v]

set_property file_type {Verilog Header} [get_files  $arm_ip_lib/Corstone-101/logical/cmsdk_apb_dualtimers/verilog/cmsdk_apb_dualtimers_defs.v]
set_property file_type {Verilog Header} [get_files  $arm_ip_lib/Corstone-101/logical/cmsdk_apb_watchdog/verilog/cmsdk_apb_watchdog_defs.v]
set_property file_type {Verilog Header} [get_files  $proj_dir/system/defines/pl230/pl230_defs.v]

add_files $fpga_target/fpga_pinmap.xdc

set_property top design_1_wrapper [current_fileset]

#
# STEP#3: save in Project mode to complete flow
#
save_project_as $project ./$project -exclude_run_results -force

update_compile_order -fileset sources_1

#
# STEP#4: synthesize project
#
set_property part $xilinx_part [get_runs synth_1]
launch_runs synth_1 -jobs 8

wait_on_run synth_1

#
# STEP#5: place and route project
#
set_property part $xilinx_part [get_runs impl_1]
launch_runs impl_1 -to_step write_bitstream -jobs 8

wait_on_run impl_1

#
# STEP#6: export design_1.bit and design_1.hwh files for PYNQ
#

write_hw_platform -fixed -include_bit -force -file $project/design_1.xsa

exec unzip -u -o $project/design_1.xsa -d $project/export

exec mkdir -p $pynqDir
exec cp -p $project/export/design_1.bit $pynqDir
exec cp -p $project/export/design_1.hwh $pynqDir
#exec cp -p $project/export/design_1.bit $exportDir
#exec cp -p $project/export/design_1.hwh $exportDir

exec rm -Rf $project/design_1.xsa
exec rm -Rf $project/export
exec rm -Rf ./vivado/

exit 1
