//-----------------------------------------------------------------------------
// SoC Labs Basic Example Accelerator Wrapper
// A joint work commissioned on behalf of SoC Labs; under Arm Academic Access license.
//
// Contributors
//
// David Mapstone (d.a.mapstone@soton.ac.uk)
// David Flynn    (d.w.flynn@soton.ac.uk)
//
// Copyright (C) 2023; SoC Labs (www.soclabs.org)
//-----------------------------------------------------------------------------

module nanosoc_exp_wrapper #(
  parameter AHBADDRWIDTH=16,
  parameter INPACKETWIDTH=128,
  parameter CFGSIZEWIDTH=64,
  parameter CFGSCHEMEWIDTH=2,
  parameter OUTPACKETWIDTH=128,
  parameter CFGNUMIRQ=4
  ) (
    input  wire                      HCLK,       // Clock
    input  wire                      HRESETn,    // Reset

    // AHB connection to Initiator
    input  wire                      HSEL_i,
    input  wire   [AHBADDRWIDTH-1:0] HADDR_i,
    input  wire   [1:0]              HTRANS_i,
    input  wire   [2:0]              HSIZE_i,
    input  wire   [3:0]              HPROT_i,
    input  wire                      HWRITE_i,
    input  wire                      HREADY_i,
    input  wire   [31:0]             HWDATA_i,

    output wire                      HREADYOUT_o,
    output wire                      HRESP_o,
    output wire   [31:0]             HRDATA_o,

    // Input Data Request Signal to DMAC
    output wire                      exp_drq_ip_o,
    input  wire                      exp_dlast_ip_i,

    // Output Data Request Signal to DMAC
    output wire                      exp_drq_op_o,
    input  wire                      exp_dlast_op_i,
    
    // Interrupts
    output wire   [CFGNUMIRQ-1:0]    exp_irq_o

  );
  
  soclabs_ahb_aes128_ctrl u_exp_aes128 (
    .ahb_hclk        (HCLK),
    .ahb_hresetn     (HRESETn),
    .ahb_hsel        (HSEL_i),
    .ahb_haddr16     (HADDR_i[15:0]),
    .ahb_htrans      (HTRANS_i),
    .ahb_hwrite      (HWRITE_i),
    .ahb_hsize       (HSIZE_i),
    .ahb_hprot       (HPROT_i),
    .ahb_hwdata      (HWDATA_i),
    .ahb_hready      (HREADY_i),
    .ahb_hrdata      (HRDATA_o),
    .ahb_hreadyout   (HREADYOUT_o),
    .ahb_hresp       (HRESP_o),
    .drq_ipdma128    (exp_drq_ip_o),
    .dlast_ipdma128  (1'b0),
    .drq_opdma128    (exp_drq_op_o),
    .dlast_opdma128  (1'b0),
    .irq_key128      (exp_irq_o[0]),
    .irq_ip128       (exp_irq_o[1]),
    .irq_op128       (exp_irq_o[2]),
    .irq_error       (exp_irq_o[3]),
    .irq_merged      ( )
  );

endmodule
